"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.create = exports.useDeviceContext = exports.plugin = void 0;
const resolveConfig_1 = __importDefault(require("tailwindcss/resolveConfig"));
const cache_1 = __importDefault(require("./cache"));
const ClassParser_1 = __importDefault(require("./ClassParser"));
const parse_inputs_1 = require("./parse-inputs");
const helpers_1 = require("./helpers");
const plugin_1 = __importStar(require("./plugin"));
exports.plugin = plugin_1.default;
const color_1 = require("./resolve/color");
var hooks_1 = require("./hooks");
Object.defineProperty(exports, "useDeviceContext", { enumerable: true, get: function () { return hooks_1.useDeviceContext; } });
function create(customConfig = {}) {
    const config = (0, resolveConfig_1.default)(customConfig);
    const device = {};
    const pluginUtils = (0, plugin_1.getAddedUtilities)(config.plugins);
    const customStringUtils = {};
    const customStyleUtils = Object.entries(pluginUtils)
        .map(([util, style]) => {
        if (typeof style === `string`) {
            // mutating while mapping, i know - bad form, but for performance sake... ¯\_(ツ)_/¯
            customStringUtils[util] = style;
            return [util, { kind: `null` }];
        }
        return [util, (0, helpers_1.complete)(style)];
    })
        .filter(([, ir]) => ir.kind !== `null`);
    function deriveCacheGroup() {
        return ([
            device.windowDimensions ? `w${device.windowDimensions.width}` : false,
            device.windowDimensions ? `h${device.windowDimensions.height}` : false,
            device.fontScale ? `fs${device.fontScale}` : false,
            device.colorScheme === `dark` ? `dark` : false,
            device.pixelDensity === 2 ? `retina` : false,
        ]
            .filter(Boolean)
            .join(`--`) || `default`);
    }
    let cacheGroup = deriveCacheGroup();
    const contextCaches = {};
    function getCache() {
        const existing = contextCaches[cacheGroup];
        if (existing) {
            return existing;
        }
        const cache = new cache_1.default(customStyleUtils);
        contextCaches[cacheGroup] = cache;
        return cache;
    }
    function style(...inputs) {
        const cache = getCache();
        let resolved = {};
        const dependents = [];
        const ordered = [];
        const [utilities, userStyle] = (0, parse_inputs_1.parseInputs)(inputs);
        // check if we've seen this full set of classes before
        // if we have a cached copy, we can skip examining each utility
        const joined = utilities.join(` `);
        const cached = cache.getStyle(joined);
        if (cached) {
            return Object.assign(Object.assign({}, cached), (userStyle ? userStyle : {}));
        }
        for (const utility of utilities) {
            let styleIr = cache.getIr(utility);
            if (!styleIr) {
                if (utility in customStringUtils) {
                    const customStyle = style(customStringUtils[utility]);
                    cache.setIr(utility, (0, helpers_1.complete)(customStyle));
                    return customStyle;
                }
                const parser = new ClassParser_1.default(utility, config, cache, device);
                styleIr = parser.parse();
            }
            switch (styleIr.kind) {
                case `complete`:
                    resolved = Object.assign(Object.assign({}, resolved), styleIr.style);
                    cache.setIr(utility, styleIr);
                    break;
                case `dependent`:
                    dependents.push(styleIr);
                    break;
                case `ordered`:
                    ordered.push(styleIr);
                    break;
                case `null`:
                    cache.setIr(utility, styleIr);
                    break;
            }
        }
        if (ordered.length > 0) {
            ordered.sort((a, b) => a.order - b.order);
            for (const orderedStyle of ordered) {
                switch (orderedStyle.styleIr.kind) {
                    case `complete`:
                        resolved = Object.assign(Object.assign({}, resolved), orderedStyle.styleIr.style);
                        break;
                    case `dependent`:
                        dependents.push(orderedStyle.styleIr);
                        break;
                }
            }
        }
        if (dependents.length > 0) {
            for (const dependent of dependents) {
                const error = dependent.complete(resolved);
                if (error) {
                    (0, helpers_1.warn)(error);
                }
            }
            (0, color_1.removeOpacityHelpers)(resolved);
        }
        // cache the full set of classes for future re-renders
        // it's important we cache BEFORE merging in userStyle below
        if (joined !== ``) {
            cache.setStyle(joined, resolved);
        }
        if (userStyle) {
            resolved = Object.assign(Object.assign({}, resolved), userStyle);
        }
        return resolved;
    }
    function color(utils) {
        const styleObj = style(utils
            .split(/\s+/g)
            .map((util) => util.replace(/^(bg|text)-/, ``))
            .map((util) => `bg-${util}`)
            .join(` `));
        return typeof styleObj.backgroundColor === `string`
            ? styleObj.backgroundColor
            : undefined;
    }
    // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
    const tailwindFn = (strings, ...values) => {
        let str = ``;
        strings.forEach((string, i) => {
            str += string + (values[i] || ``);
        });
        return style(str);
    };
    tailwindFn.style = style;
    tailwindFn.color = color;
    tailwindFn.prefixMatch = (...prefixes) => {
        const joined = prefixes.sort().join(`:`);
        const cache = getCache();
        const cached = cache.getPrefixMatch(joined);
        if (cached !== undefined) {
            return cached;
        }
        const parser = new ClassParser_1.default(`${joined}:flex`, config, cache, device);
        const ir = parser.parse();
        const prefixMatches = ir.kind !== `null`;
        cache.setPrefixMatch(joined, prefixMatches);
        return prefixMatches;
    };
    tailwindFn.setWindowDimensions = (newDimensions) => {
        device.windowDimensions = newDimensions;
        cacheGroup = deriveCacheGroup();
    };
    tailwindFn.setFontScale = (newFontScale) => {
        device.fontScale = newFontScale;
        cacheGroup = deriveCacheGroup();
    };
    tailwindFn.setPixelDensity = (newPixelDensity) => {
        device.pixelDensity = newPixelDensity;
        cacheGroup = deriveCacheGroup();
    };
    tailwindFn.setColorScheme = (newColorScheme) => {
        device.colorScheme = newColorScheme;
        cacheGroup = deriveCacheGroup();
    };
    return tailwindFn;
}
exports.create = create;
const tailwind = create();
exports.default = tailwind;
