"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveTailwindConfig = void 0;
const resolveConfig_1 = __importDefault(require("tailwindcss/resolveConfig"));
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const findConfigFilePath = (currentDir) => {
    const files = fs_1.default.readdirSync(currentDir);
    const configFile = files.find((file) => file === "tailwind.config.js");
    if (configFile)
        return path_1.default.resolve(currentDir, configFile);
    // Check if package.json exists on this level, in which case we conclude that there is no config file
    const packageJson = files.find((file) => file === "package.json");
    if (packageJson)
        return null;
    // Otherwise, recursively move upwards
    return findConfigFilePath(path_1.default.resolve(currentDir, ".."));
};
const resolveTailwindConfig = (filePath) => {
    if (!filePath)
        return (0, resolveConfig_1.default)({ theme: {}, darkMode: "media" });
    const configFilePath = fs_1.default.lstatSync(filePath).isDirectory()
        ? findConfigFilePath(filePath)
        : findConfigFilePath(path_1.default.dirname(filePath));
    const config = configFilePath
        ? require(configFilePath)
        : { theme: {}, darkMode: "media" };
    return (0, resolveConfig_1.default)(config);
};
exports.resolveTailwindConfig = resolveTailwindConfig;
